#pragma once
#include <memory>
#include <string>
#include <stdexcept>
namespace KapitanGame {
	namespace Utils {

		template<typename ... Args>
		std::string StringFormat(const std::string& format, Args ... args)
		{
			const int sizeS = std::snprintf(nullptr, 0, format.c_str(), args ...) + 1; // Extra space for '\0'
			if (sizeS <= 0) { throw std::runtime_error("Error during formatting."); }
			const auto size = static_cast<size_t>(sizeS);
			const auto buf = std::make_unique<char[]>(size);
			std::snprintf(buf.get(), size, format.c_str(), args ...);
			return std::string(buf.get(), buf.get() + size - 1); // We don't want the '\0' inside
		}
		template <class T>
		constexpr const T& Clamp(const T& x)
		{
			return std::max(std::min(1.f, x), 0.f);
		}

		template <class T>
		constexpr const T& Lerp(const T& a, const T& b, const float& t)
		{
			return a + (b - a) * t;
		}
	}
}